@@ -27,8 +27,10 @@
 #include <QQmlExpression>
 #include <QQmlProperty>
 #include <QMimeData>
+#include <QVersionNumber>
 
 #include <kactioncollection.h>
+#include <KAcceleratorManager>
 #include <kauthorized.h>
 #include <QDebug>
 #include <klocalizedstring.h>
@@ -40,6 +42,7 @@
 #include "kio/jobclasses.h" // for KIO::JobFlags
 #include "kio/job.h"
 #include "kio/scheduler.h"
+#include <KIO/DropJob>
 #endif
 
 #include <plasma.h>
@@ -59,7 +62,8 @@ ContainmentInterface::ContainmentInterface(DeclarativeAppletScript *parent, cons
     : AppletInterface(parent, args),
       m_wallpaperInterface(0),
       m_activityInfo(0),
-      m_wheelDelta(0)
+      m_wheelDelta(0),
+      m_editMode(false)
 {
     m_containment = static_cast<Plasma::Containment *>(appletScript()->applet()->containment());
 
@@ -92,9 +96,9 @@ void ContainmentInterface::init()
     if (m_containment) {
         KConfigGroup defaults;
         if (m_containment->containmentType() == Plasma::Types::DesktopContainment) {
-            defaults = KConfigGroup(KSharedConfig::openConfig(m_containment->corona()->package().filePath("defaults")), "Desktop");
+            defaults = KConfigGroup(KSharedConfig::openConfig(m_containment->corona()->kPackage().filePath("defaults")), "Desktop");
         } else if (m_containment->containmentType() == Plasma::Types::PanelContainment) {
-            defaults = KConfigGroup(KSharedConfig::openConfig(m_containment->corona()->package().filePath("defaults")), "Panel");
+            defaults = KConfigGroup(KSharedConfig::openConfig(m_containment->corona()->kPackage().filePath("defaults")), "Panel");
         }
 
         if (defaults.isValid()) {
@@ -190,40 +194,6 @@ void ContainmentInterface::setContainmentType(Plasma::Types::ContainmentType typ
     appletScript()->setContainmentType(type);
 }
 
-QVariantList ContainmentInterface::availableScreenRegion() const
-{
-    QRegion reg = QRect(0, 0, width(), height());
-    int screenId = screen();
-    if (screenId > -1 && m_containment->corona()) {
-        reg = m_containment->corona()->availableScreenRegion(screenId);
-    }
-
-    QVariantList regVal;
-    foreach (QRect rect, reg.rects()) {
-        //make it relative
-        QRect geometry = m_containment->corona()->screenGeometry(screenId);
-        rect.moveTo(rect.topLeft() - geometry.topLeft());
-        regVal << QVariant::fromValue(QRectF(rect));
-    }
-    return regVal;
-}
-
-QRect ContainmentInterface::availableScreenRect() const
-{
-    QRect rect(0, 0, width(), height());
-
-    int screenId = screen();
-
-    if (screenId > -1 && m_containment->corona()) {
-        rect = m_containment->corona()->availableScreenRect(screenId);
-        //make it relative
-        QRect geometry = m_containment->corona()->screenGeometry(screenId);
-        rect.moveTo(rect.topLeft() - geometry.topLeft());
-    }
-
-    return rect;
-}
-
 Plasma::Applet *ContainmentInterface::createApplet(const QString &plugin, const QVariantList &args, const QPoint &pos)
 {
     return createApplet(plugin, args, QRectF(pos, QSize()));
@@ -244,8 +214,7 @@ Plasma::Applet *ContainmentInterface::createApplet(const QString &plugin, const
             return applet;
         }
         if (geom.width() > 0 && geom.height() > 0) {
-            appletGraphicObject->setWidth(geom.width());
-            appletGraphicObject->setHeight(geom.height());
+            appletGraphicObject->setSize(geom.size());
         }
 
         blockSignals(false);
@@ -272,7 +241,7 @@ void ContainmentInterface::setAppletArgs(Plasma::Applet *applet, const QString &
 
 QObject *ContainmentInterface::containmentAt(int x, int y)
 {
-    QObject *desktop = Q_NULLPTR;
+    QObject *desktop = nullptr;
     foreach (Plasma::Containment *c, m_containment->corona()->containments()) {
         ContainmentInterface *contInterface = c->property("_plasma_graphicObject").value<ContainmentInterface *>();
 
@@ -442,17 +411,32 @@ QPointF ContainmentInterface::adjustToAvailableScreenRegion(int x, int y, int w,
     return rect.topLeft();
 }
 
-void ContainmentInterface::processMimeData(QObject *mimeDataProxy, int x, int y)
+bool ContainmentInterface::isEditMode() const
+{
+    return m_editMode;
+}
+
+void ContainmentInterface::setEditMode(bool edit)
+{
+    if (edit == m_editMode) {
+        return;
+    }
+
+    m_editMode = edit;
+    emit editModeChanged();
+}
+
+void ContainmentInterface::processMimeData(QObject *mimeDataProxy, int x, int y, KIO::DropJob *dropJob)
 {
     QMimeData* mime = qobject_cast<QMimeData*>(mimeDataProxy);
     if (mime) {
-        processMimeData(mime, x, y);
+        processMimeData(mime, x, y, dropJob);
     } else {
-        processMimeData(mimeDataProxy->property("mimeData").value<QMimeData*>(), x, y);
+        processMimeData(mimeDataProxy->property("mimeData").value<QMimeData*>(), x, y, dropJob);
     }
 }
 
-void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y)
+void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KIO::DropJob *dropJob)
 {
     if (!mimeData) {
         return;
@@ -493,11 +477,15 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y)
             QObject::connect(job, SIGNAL(mimetype(KIO::Job*,QString)),
                              this, SLOT(mimeTypeRetrieved(KIO::Job*,QString)));
 
-            QMenu *choices = new QMenu(i18n("Content dropped"));
-            choices->addAction(QIcon::fromTheme(QStringLiteral("process-working")), i18n("Fetching file type..."));
-            choices->popup(window() ? window()->mapToGlobal(QPoint(x, y)) : QPoint(x, y));
+            if (dropJob) {
+                m_dropJobs[job] = dropJob;
+            } else {
+                QMenu *choices = new QMenu(i18n("Content dropped"));
+                choices->addAction(QIcon::fromTheme(QStringLiteral("process-working")), i18n("Fetching file type..."));
+                choices->popup(window() ? window()->mapToGlobal(QPoint(x, y)) : QPoint(x, y));
 
-            m_dropMenus[job] = choices;
+                m_dropMenus[job] = choices;
+            }
 #endif
         }
 
@@ -524,34 +512,49 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y)
 
         if (seenPlugins.isEmpty()) {
             // do nothing
+        //directly create if only one offer only if the conteinment didn't pass an existing plugin
         } else if (seenPlugins.count() == 1) {
             selectedPlugin = seenPlugins.constBegin().key();
+            Plasma::Applet *applet = createApplet(selectedPlugin, QVariantList(), QRect(x, y, -1, -1));
+            setAppletArgs(applet, pluginFormats[selectedPlugin], mimeData->data(pluginFormats[selectedPlugin]));
         } else {
-            QMenu choices;
+            QMenu *choices = nullptr;
+            if (!dropJob) {
+                choices = new QMenu();
+            }
+            QList<QAction *> extraActions;
             QHash<QAction *, QString> actionsToPlugins;
             foreach (const KPluginInfo &info, seenPlugins) {
                 QAction *action;
                 if (!info.icon().isEmpty()) {
-                    action = choices.addAction(QIcon::fromTheme(info.icon()), info.name());
+                    action = new QAction(QIcon::fromTheme(info.icon()), info.name(), nullptr);
                 } else {
-                    action = choices.addAction(info.name());
+                    action = new QAction(info.name(), nullptr);
                 }
+                extraActions << action;
+                if (choices) {
+                    choices->addAction(action);
+                }
+                action->setData(info.pluginName());
+                connect(action, &QAction::triggered, this, [this, x, y, mimeData, action]() {
+                    const QString selectedPlugin = action->data().toString();
+                    Plasma::Applet *applet = createApplet(selectedPlugin, QVariantList(), QRect(x, y, -1, -1));
+                    setAppletArgs(applet, selectedPlugin, mimeData->data(selectedPlugin));
+                });
 
                 actionsToPlugins.insert(action, info.pluginName());
             }
 
-            QAction *choice = choices.exec(window() ? window()->mapToGlobal(QPoint(x, y)) : QPoint(x, y));
-            if (choice) {
-                selectedPlugin = actionsToPlugins[choice];
+            //if the menu was created by ourselves, delete it
+            if (choices) {
+                QAction *choice = choices->exec(window() ? window()->mapToGlobal(QPoint(x, y)) : QPoint(x, y));
+
+                delete choices;
+            } else {
+                Q_ASSERT(dropJob);
+                dropJob->setApplicationActions(extraActions);
             }
         }
-
-        if (!selectedPlugin.isEmpty()) {
-
-            Plasma::Applet *applet = createApplet(selectedPlugin, QVariantList(), QRect(x, y, -1, -1));
-            setAppletArgs(applet, pluginFormats[selectedPlugin], mimeData->data(pluginFormats[selectedPlugin]));
-
-        }
     }
 }
 
@@ -561,7 +564,7 @@ void ContainmentInterface::clearDataForMimeJob(KIO::Job *job)
     QObject::disconnect(job, 0, this, 0);
     m_dropPoints.remove(job);
     QMenu *choices = m_dropMenus.take(job);
-    delete choices;
+    m_dropJobs.remove(job);
     job->kill();
 #endif // PLASMA_NO_KIO
 }
@@ -572,9 +575,6 @@ void ContainmentInterface::dropJobResult(KJob *job)
     if (job->error()) {
         qDebug() << "ERROR" << job->error() << ' ' << job->errorString();
     }
-    // We call mimetypeRetrieved since there might be other mechanisms
-    // for finding suitable applets. Cleanup happens there as well.
-    mimeTypeRetrieved(qobject_cast<KIO::Job *>(job), QString());
 #endif // PLASMA_NO_KIO
 }
 
@@ -582,6 +582,7 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
 {
 #ifndef PLASMA_NO_KIO
     qDebug() << "Mimetype Job returns." << mimetype;
+
     KIO::TransferJob *tjob = dynamic_cast<KIO::TransferJob *>(job);
     if (!tjob) {
         qDebug() << "job should be a TransferJob, but isn't";
@@ -596,8 +597,8 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
     } else {
 
         QPoint posi; // will be overwritten with the event's position
-        if (m_dropPoints.keys().contains(tjob)) {
-            posi = m_dropPoints[tjob];
+        if (m_dropPoints.contains(tjob)) {
+            posi = m_dropPoints.value(tjob);
             qDebug() << "Received a suitable dropEvent at" << posi;
         } else {
             qDebug() << "Bailing out. Cannot find associated dropEvent related to the TransferJob";
@@ -606,8 +607,11 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
         }
 
         QMenu *choices = m_dropMenus.value(tjob);
-        if (!choices) {
-            qDebug() << "Bailing out. No QMenu found for this job.";
+        QList<QAction *> dropActions;
+        KIO::DropJob *dropJob = m_dropJobs.value(tjob);
+
+        if (!choices && !dropJob) {
+            qDebug() << "Bailing out. No QMenu or drop job found for this job.";
             clearDataForMimeJob(job);
             return;
         }
@@ -633,71 +637,25 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
         if (!appletList.isEmpty() || !wallpaperList.isEmpty() || isPlasmaPackage) {
             QAction *installPlasmaPackageAction = nullptr;
             if (isPlasmaPackage) {
-                choices->addSection(i18n("Plasma Package"));
-                installPlasmaPackageAction = choices->addAction(QIcon::fromTheme(QStringLiteral("application-x-plasma")), i18n("Install"));
-            }
-
-            QHash<QAction *, QString> actionsToApplets;
-            choices->addSection(i18n("Widgets"));
-            foreach (const KPluginInfo &info, appletList) {
-                qDebug() << info.name();
-                QAction *action;
-                if (!info.icon().isEmpty()) {
-                    action = choices->addAction(QIcon::fromTheme(info.icon()), info.name());
+                if (choices) {
+                    choices->addSection(i18n("Plasma Package"));
+                    installPlasmaPackageAction = choices->addAction(QIcon::fromTheme(QStringLiteral("application-x-plasma")), i18n("Install"));
                 } else {
-                    action = choices->addAction(info.name());
+                    QAction *action = new QAction(i18n("Plasma Package"), nullptr);
+                    action->setSeparator(true);
+                    dropActions << action;
+
+                    installPlasmaPackageAction = new QAction(QIcon::fromTheme(QStringLiteral("application-x-plasma")), i18n("Install"), nullptr);
+                    Q_ASSERT(dropJob);
+                    dropActions << installPlasmaPackageAction;
+                    dropJob->setApplicationActions(dropActions);
                 }
-
-                actionsToApplets.insert(action, info.pluginName());
-                qDebug() << info.pluginName();
-            }
-            actionsToApplets.insert(choices->addAction(i18n("Icon")), QStringLiteral("org.kde.plasma.icon"));
-
-            QHash<QAction *, QString> actionsToWallpapers;
-            if (!wallpaperList.isEmpty())  {
-                choices->addSection(i18n("Wallpaper"));
-
-                QMap<QString, KPluginInfo> sorted;
-                foreach (const KPluginInfo &info, appletList) {
-                    sorted.insert(info.name(), info);
-                }
-
-                foreach (const KPluginInfo &info, wallpaperList) {
-                    QAction *action;
-                    if (!info.icon().isEmpty()) {
-                        action = choices->addAction(QIcon::fromTheme(info.icon()), info.name());
-                    } else {
-                        action = choices->addAction(info.name());
-                    }
-
-                    actionsToWallpapers.insert(action, info.pluginName());
-                }
-            }
-
-            // HACK If the QMenu becomes empty at any point after the "determining mimetype"
-            // popup was shown, it self-destructs, does not matter if we call clear() or remove
-            // the action manually, hence we remove the aforementioned item after we populated the menu
-            choices->removeAction(choices->actions().at(0));
-
-            QAction *choice = choices->exec();
-            if (choice) {
-                // Put the job on hold so it can be recycled to fetch the actual content,
-                // which is to be expected when something's dropped onto the desktop and
-                // an applet is to be created with this URL
-                if (!mimetype.isEmpty() && !tjob->error()) {
-                    tjob->putOnHold();
-                    KIO::Scheduler::publishSlaveOnHold();
-                }
-
-                QString plugin = actionsToApplets.value(choice);
-
-                if (choice == installPlasmaPackageAction) {
+                const QString &packagePath = tjob->url().toLocalFile();
+                connect(installPlasmaPackageAction, &QAction::triggered, this, [this, tjob, posi, packagePath]() {
                     using namespace KPackage;
                     PackageStructure *structure = PackageLoader::self()->loadPackageStructure(QStringLiteral("Plasma/Applet"));
                     Package package(structure);
 
-                    const QString &packagePath = tjob->url().toLocalFile();
-
                     KJob *installJob = package.update(packagePath);
                     connect(installJob, &KJob::result, this, [this, packagePath, structure, posi](KJob *job) {
                         auto fail = [job](const QString &text) {
@@ -716,6 +674,7 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
                         using namespace KPackage;
                         Package package(structure);
                         // TODO how can I get the path of the actual package?
+
                         package.setPath(packagePath);
 
                         // TODO how can I get the plugin id? Package::metadata() is deprecated
@@ -726,28 +685,128 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
 
                         createApplet(package.metadata().pluginId(), QVariantList(), QRect(posi, QSize(-1,-1)));
                     });
-                } else if (plugin.isEmpty()) {
-                    //set wallpapery stuff
-                    plugin = actionsToWallpapers.value(choice);
-                    if (m_wallpaperInterface && tjob->url().isValid()) {
-                        m_wallpaperInterface->setUrl(tjob->url());
-                    }
+                });
+            }
+
+            if (choices) {
+                choices->addSection(i18n("Widgets"));
+            } else {
+                QAction *action = new QAction(i18n("Widgets"), nullptr);
+                action->setSeparator(true);
+                dropActions << action;
+            }
+            foreach (const KPluginInfo &info, appletList) {
+                qDebug() << info.name();
+                QAction *action;
+                if (!info.icon().isEmpty()) {
+                    action = new QAction(QIcon::fromTheme(info.icon()), info.name(), nullptr);
                 } else {
-                    Plasma::Applet *applet = createApplet(actionsToApplets[choice], QVariantList(), QRect(posi, QSize(-1,-1)));
-                    setAppletArgs(applet, mimetype, tjob->url().toString());
+                    action = new QAction(info.name(), nullptr);
                 }
 
-                clearDataForMimeJob(job);
-                return;
+                if (choices) {
+                    choices->addAction(action);
+                }
+                dropActions << action;
+                action->setData(info.pluginName());
+                qDebug() << info.pluginName();
+                const QUrl url = tjob->url();
+                connect(action, &QAction::triggered, this, [this, action, posi, mimetype, url]() {
+                    Plasma::Applet *applet = createApplet(action->data().toString(), QVariantList(), QRect(posi, QSize(-1,-1)));
+                    setAppletArgs(applet, mimetype, url.toString());
+                });
+            }
+            {
+                QAction *action = new QAction(i18n("Icon"), nullptr);
+                if (choices) {
+                    choices->addAction(action);
+                }
+                dropActions << action;
+                action->setData(QStringLiteral("org.kde.plasma.icon"));
+                const QUrl url = tjob->url();
+                connect(action, &QAction::triggered, this, [this, action, posi, mimetype, url](){
+                    Plasma::Applet *applet = createApplet(action->data().toString(), QVariantList(), QRect(posi, QSize(-1,-1)));
+                    setAppletArgs(applet, mimetype, url.toString());
+                });
             }
-        } else {
-            // we can at least create an icon as a link to the URL
-            Plasma::Applet *applet = createApplet(QStringLiteral("org.kde.plasma.icon"), QVariantList(), QRect(posi, QSize(-1,-1)));
-            setAppletArgs(applet, mimetype, tjob->url().toString());
-        }
-    }
 
-    clearDataForMimeJob(job);
+            QHash<QAction *, QString> actionsToWallpapers;
+            if (!wallpaperList.isEmpty())  {
+                if (choices) {
+                    choices->addSection(i18n("Wallpaper"));
+                } else {
+                    QAction *action = new QAction(i18n("Wallpaper"), nullptr);
+                    action->setSeparator(true);
+                    dropActions << action;
+                }
+
+                QMap<QString, KPluginInfo> sorted;
+                foreach (const KPluginInfo &info, appletList) {
+                    sorted.insert(info.name(), info);
+                }
+
+                foreach (const KPluginInfo &info, wallpaperList) {
+                    QAction *action;
+                    if (!info.icon().isEmpty()) {
+                        action = new QAction(QIcon::fromTheme(info.icon()), info.name(), nullptr);
+                    } else {
+                        action = new QAction(info.name(), nullptr);
+                    }
+
+                    if (choices) {
+                        choices->addAction(action);
+                    }
+                    dropActions << action;
+                    actionsToWallpapers.insert(action, info.pluginName());
+                    const QUrl url = tjob->url();
+                    connect(action, &QAction::triggered, this, [this, action, url]() {
+                        //set wallpapery stuff
+                        if (m_wallpaperInterface && url.isValid()) {
+                            m_wallpaperInterface->setUrl(url);
+                        }
+                    });
+                }
+            }
+
+        } else {
+            //case in which we created the menu ourselves, just the "fetching type entry, directly create the icon applet
+            if (choices) {
+                Plasma::Applet *applet = createApplet(QStringLiteral("org.kde.plasma.icon"), QVariantList(), QRect(posi, QSize(-1,-1)));
+                setAppletArgs(applet, mimetype, tjob->url().toString());
+            } else {
+                QAction *action;
+                if (choices) {
+                    choices->addSection(i18n("Widgets"));
+                    action = choices->addAction(i18n("Icon"));
+                } else {
+                    QAction *sep = new QAction(i18n("Widgets"), nullptr);
+                    sep->setSeparator(true);
+                    dropActions << sep;
+                    // we can at least create an icon as a link to the URL
+                    action = new QAction(i18n("Icon"), nullptr);
+                    dropActions << action;
+                }
+
+                const QUrl url = tjob->url();
+                connect(action, &QAction::triggered, this, [this, posi, mimetype, url](){
+                    Plasma::Applet *applet = createApplet(QStringLiteral("org.kde.plasma.icon"), QVariantList(), QRect(posi, QSize(-1,-1)));
+                    setAppletArgs(applet, mimetype, url.toString());
+                });
+            }
+        }
+
+        if (choices) {
+            // HACK If the QMenu becomes empty at any point after the "determining mimetype"
+            // popup was shown, it self-destructs, does not matter if we call clear() or remove
+            // the action manually, hence we remove the aforementioned item after we populated the menu
+            choices->removeAction(choices->actions().at(0));
+            choices->exec();
+        } else {
+            dropJob->setApplicationActions(dropActions);
+        }
+
+        clearDataForMimeJob(tjob);
+    }
 #endif // PLASMA_NO_KIO
 }
 
@@ -763,7 +822,8 @@ void ContainmentInterface::appletAddedForward(Plasma::Applet *applet)
 //     qDebug() << "Applet added on containment:" << m_containment->title() << contGraphicObject
 //              << "Applet: " << applet << applet->title() << appletGraphicObject;
 
-    //Every applet should have a graphics object, otherwise don't disaplay anything
+    //applets can not have a graphic object if they don't have a script engine loaded
+    //this can happen if they were loaded with an invalid metadata
     if (!appletGraphicObject) {
         return;
     }
@@ -785,8 +845,10 @@ void ContainmentInterface::appletAddedForward(Plasma::Applet *applet)
 void ContainmentInterface::appletRemovedForward(Plasma::Applet *applet)
 {
     AppletInterface *appletGraphicObject = applet->property("_plasma_graphicObject").value<AppletInterface *>();
-    m_appletInterfaces.removeAll(appletGraphicObject);
-    appletGraphicObject->m_positionBeforeRemoval = appletGraphicObject->mapToItem(this, QPointF());
+    if (appletGraphicObject) {
+        m_appletInterfaces.removeAll(appletGraphicObject);
+        appletGraphicObject->m_positionBeforeRemoval = appletGraphicObject->mapToItem(this, QPointF());
+    }
     emit appletRemoved(appletGraphicObject);
     emit appletsChanged();
 }
@@ -954,12 +1016,33 @@ void ContainmentInterface::mousePressEvent(QMouseEvent *event)
     desktopMenu->setAttribute(Qt::WA_TranslucentBackground);
     //end workaround
 
-    if (window() && window()->mouseGrabberItem()) {
-        window()->mouseGrabberItem()->ungrabMouse();
+
+    //this is a workaround where Qt will fail to realise a mouse has been released
+
+    // this happens if a window which does not accept focus spawns a new window that takes focus and X grab
+    // whilst the mouse is depressed
+    // https://bugreports.qt.io/browse/QTBUG-59044
+    // this causes the next click to go missing
+
+    //by releasing manually we avoid that situation
+    auto ungrabMouseHack = [this]() {
+        if (window() && window()->mouseGrabberItem()) {
+            window()->mouseGrabberItem()->ungrabMouse();
+        }
+    };
+
+    //pre 5.8.0 QQuickWindow code is "item->grabMouse(); sendEvent(item, mouseEvent)"
+    //post 5.8.0 QQuickWindow code is sendEvent(item, mouseEvent); item->grabMouse()
+    if (QVersionNumber::fromString(qVersion()) > QVersionNumber(5, 8, 0)) {
+        QTimer::singleShot(0, this, ungrabMouseHack);
     }
+    else {
+        ungrabMouseHack();
+    }
+    //end workaround
 
     QPoint pos = event->globalPos();
-    if (window() && applet && m_containment->containmentType() == Plasma::Types::PanelContainment) {
+    if (window() && m_containment->containmentType() == Plasma::Types::PanelContainment) {
         desktopMenu->adjustSize();
 
         if (QScreen *screen = window()->screen()) {
@@ -976,6 +1059,8 @@ void ContainmentInterface::mousePressEvent(QMouseEvent *event)
         return;
     }
 
+    KAcceleratorManager::manage(desktopMenu);
+
     desktopMenu->popup(pos);
     event->setAccepted(true);
 }
@@ -1004,6 +1089,17 @@ void ContainmentInterface::wheelEvent(QWheelEvent *event)
     }
 }
 
+void ContainmentInterface::keyPressEvent(QKeyEvent *event)
+{
+    if (event->key() == Qt::Key_Menu) {
+        QMouseEvent me(QEvent::MouseButtonRelease, QPoint(), Qt::RightButton, Qt::RightButton, event->modifiers());
+        mousePressEvent(&me);
+        event->accept();
+    }
+
+    AppletInterface::keyPressEvent(event);
+}
+
 void ContainmentInterface::addAppletActions(QMenu *desktopMenu, Plasma::Applet *applet, QEvent *event)
 {
     foreach (QAction *action, applet->contextualActions()) {
